==> XMLObject-0.1.0.tar.gz: XMLObject-0.1.0/XMLObject/PrettyXMLPrinter.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)

import re

class NotWellFormedXML(Exception):
    def __init__(self, misMatched, expected):
        self.misMatched = misMatched
        self.expected = expected
        
    def __str__(self):
        return 'Expected an "%s" closing tag before the "%s" tag.' % (self.expected,
                                                                      self.misMatched)

    __repr__ = __str__

class NoCallbackError(Exception):
    def  __init__(self, xmlData):
        self.xml = xmlData

    def __str__(self):
        return "Don't know how to handle '%s'" % self.xml

    __repr__ = __str__
    
class PrettyXMLPrinter:
    regexps = { 'content': re.compile('^[^<]'),
                'cdata': re.compile('<\!\[CDATA\[(.*)\]\]>'),
                'processing_instruction': re.compile('<\?(.*)\?>'),
                'begin_tag': re.compile('<([a-zA-Z0-9_]*)(\s+[^<$/]*)?(/)?>'),
                'end_tag': re.compile('</([a-zA-Z0-9_]*)>') }
    
    def __init__(self):
        self._reset()

    def _reset(self):
        self._tags = []

    def _buildTag(self, tagName, attrs, alone=False):
        tag = '<%s' % tagName
        if attrs:
            tag += attrs
        if alone:
            tag += '/'
        tag += '>'
        return tag
        
    def prettyPrint(self, stringToProcess, indent=' '):
        result = ''
        stringToProcess = stringToProcess.strip()
        self._indent = indent
        while stringToProcess:
            backup = stringToProcess
            for reName, regexp in self.regexps.iteritems():
                reResult = regexp.match(stringToProcess)
                if reResult:
                    callback = getattr(self, 'handle_%s' % reName)
                    toAppend, stringToProcess = callback(stringToProcess,
                                                         reResult.groups())
                    result += toAppend
                    break
            if backup == stringToProcess:
                raise NoCallbackError(stringToProcess)
        self._reset()
        return result[:-1]

    def handle_processing_instruction(self, stringToProcess, reGroups):
        " Processing Instruction "
        content = reGroups[0]
        pi = '<?%s?>' % content
        stringToProcess = stringToProcess[len(pi):]
        return pi + '\n', stringToProcess

    def handle_begin_tag(self, stringToProcess, reGroups):
        " Begin Tag "
        tagName, attrs, alone = reGroups
        tag = self._buildTag(tagName, attrs, alone)                    
        result = self._indent * len(self._tags) + tag + '\n'
        stringToProcess = stringToProcess[len(tag):]
        if not alone:
            self._tags.append(tagName)
        return result, stringToProcess

    def handle_content(self, stringToProcess, reGroups):
        " Element's content "
        nextTagIndex = stringToProcess.find('<')
        content = stringToProcess[:nextTagIndex]
        result = ''
        stringToProcess = stringToProcess[nextTagIndex:]
        content = content.strip()
        if content:
            result = self._indent * (len(self._tags) + 1) + content + '\n'
        return result, stringToProcess

    def handle_cdata(self, stringToProcess, reGroups):
        " CDATA section "
        end = stringToProcess.find(']>') + 2
        result = self._indent * (len(self._tags) + 1) + stringToProcess[:end] + '\n'
        stringToProcess = stringToProcess[end:]
        return result, stringToProcess
    
    def handle_end_tag(self, stringToProcess, reGroups):
        " Closing Tag "
        tagName = reGroups[0]
        tag = '</%s>' % tagName
        lastTag = self._tags[-1]
        if lastTag == tagName:
            self._tags = self._tags[:-1]
        else:
            raise NotWellFormedXML(tagName, lastTag)
        result = self._indent * len(self._tags) + tag + '\n'
        stringToProcess = stringToProcess[len(tag):]
        return result, stringToProcess

if __name__ == '__main__':
    s = """\
<blah><item>item1</item><blah><item>item1</item></blah></blah>
"""
    s2 = """\
<blah><item>item1<blah><item>item1</item></blah></blah>
"""

    expected = """\
<blah>
 <item>
   item1
 </item>
 <blah>
  <item>
    item1
  </item>
 </blah>
</blah>"""

    pr = PrettyXMLPrinter()
    result = pr.prettyPrint(s)
    print result
    assert result == expected, 'Pb'

    print '-' * 80
    try:
        result = pr.prettyPrint(s2)
    except Exception,e:
        print e

==> XMLObject-0.1.1.tar.gz: XMLObject-0.1.1/XMLObject/PrettyXMLPrinter.py <==
# Copyright (C) 2004 Philippe Normand <phil@respyre.org>
#
# This file is part of XMLObject (http://xmlobject.base-art.net)
#
# Under PSF License (see COPYING)

import re

class NotWellFormedXML(Exception):
    def __init__(self, misMatched, expected):
        self.misMatched = misMatched
        self.expected = expected
        
    def __str__(self):
        return 'Expected an "%s" closing tag before the "%s" tag.' % (self.expected,
                                                                      self.misMatched)

    __repr__ = __str__

class NoCallbackError(Exception):
    def  __init__(self, xmlData):
        self.xml = xmlData

    def __str__(self):
        return "Don't know how to handle '%s'" % self.xml

    __repr__ = __str__
    
class PrettyXMLPrinter:
    regexps = [ ('processing_instruction', re.compile('<\?(.*)\?>')),
                ('cdata', re.compile('<\!\[CDATA\[(.*)\]\]>')),
                ('comment', re.compile('<\!--(.*)-->')),
                ('begin_tag', re.compile('<([-a-zA-Z0-9_]*)(\s+[^<$/]*)?(/)?>')),
                ('end_tag', re.compile('</([-a-zA-Z0-9_]*)>') ),
                ('content', re.compile('^[^<]')),
                ]
    
    def __init__(self):
        self._reset()

    def _reset(self):
        self._tags = []

    def _buildTag(self, tagName, attrs, alone=False):
        tag = '<%s' % tagName
        if attrs:
            tag += attrs
        if alone:
            tag += '/'
        tag += '>'
        return tag
        
    def prettyPrint(self, stringToProcess, indent=' '):
        result = ''
        stringToProcess = stringToProcess.strip()
        self._indent = indent
        while stringToProcess:
            backup = stringToProcess
            for reName, regexp in self.regexps:
                reResult = regexp.match(stringToProcess)
                if reResult:
                    callback = getattr(self, 'handle_%s' % reName)
                    toAppend, stringToProcess = callback(stringToProcess,
                                                         reResult.groups())
                    result += toAppend
                    break
            if backup == stringToProcess:
                raise NoCallbackError(stringToProcess)
        self._reset()
        return result[:-1]

    def handle_processing_instruction(self, stringToProcess, reGroups):
        " Processing Instruction "
        content = reGroups[0]
        pi = '<?%s?>' % content
        stringToProcess = stringToProcess[len(pi):]
        return pi + '\n', stringToProcess

    def handle_comment(self, stringToProcess, reGroups):
        " Comment "
        content = '<!--%s-->' % reGroups[0]
        end = stringToProcess.find('-->') + 3
        result = self._indent * (len(self._tags) + 1) + content + '\n'
        stringToProcess = stringToProcess[end:]
        return result, stringToProcess
        

    def handle_begin_tag(self, stringToProcess, reGroups):
        " Begin Tag "
        tagName, attrs, alone = reGroups
        tag = self._buildTag(tagName, attrs, alone)                    
        result = self._indent * len(self._tags) + tag + '\n'
        stringToProcess = stringToProcess[len(tag):]
        if not alone:
            self._tags.append(tagName)
        return result, stringToProcess

    def handle_content(self, stringToProcess, reGroups):
        " Element's content "
        nextTagIndex = stringToProcess.find('<')
        content = stringToProcess[:nextTagIndex]
        result = ''
        stringToProcess = stringToProcess[nextTagIndex:]
        content = content.strip()
        if content:
            result = self._indent * (len(self._tags) + 1) + content + '\n'
        return result, stringToProcess

    def handle_cdata(self, stringToProcess, reGroups):
        " CDATA section "
        end = stringToProcess.find(']>') + 2
        result = self._indent * (len(self._tags) + 1) + stringToProcess[:end] + '\n'
        stringToProcess = stringToProcess[end:]
        return result, stringToProcess
    
    def handle_end_tag(self, stringToProcess, reGroups):
        " Closing Tag "
        tagName = reGroups[0]
        tag = '</%s>' % tagName
        lastTag = self._tags[-1]
        if lastTag == tagName:
            self._tags = self._tags[:-1]
        else:
            raise NotWellFormedXML(tagName, lastTag)
        result = self._indent * len(self._tags) + tag + '\n'
        stringToProcess = stringToProcess[len(tag):]
        return result, stringToProcess

if __name__ == '__main__':
    s = """\
<blah><item>item1</item><blah><item>item1</item></blah></blah>
"""
    s2 = """\
<blah><item>item1<blah><item>item1</item></blah></blah>
"""

    expected = """\
<blah>
 <item>
   item1
 </item>
 <blah>
  <item>
    item1
  </item>
 </blah>
</blah>"""

    pr = PrettyXMLPrinter()
    result = pr.prettyPrint(s)
    print result
    assert result == expected, 'Pb'

    print '-' * 80
    try:
        result = pr.prettyPrint(s2)
    except Exception,e:
        print e
